//###################################################################################
// ~ International Laser Display Association ~
// ~     ILDA File Format Driver in C++      ~
// ~             Built for EZFB              ~
//
// by James Lehman
// Extra Stimulus Inc.
// james@akrobiz.com
//
// began: October 2003
//
//###################################################################################
#ifndef __EZ_ILDA_FRAME_VECTOR_DEFINITIONS__
#define __EZ_ILDA_FRAME_VECTOR_DEFINITIONS__

//###################################################################################
#include "ez_ilda_frame.hpp"

//###################################################################################
class ez_ilda_frame_set : public vector<ez_ilda_frame>
{
public:
    ez_ilda_frame_set()
                     : file_name          (             ),
                       num_2D_frames      ( 0           ),
                       num_3D_frames      ( 0           ),
                       frame_index        ( 0           ),
                       frame_select_start (-1           ),
                       frame_select_end   (-1           ),
                       error              ( EZ_ILDA_OK  ),
                       null_frame         ( NULL_frame())     {}
    //-------------------------------------------------------------------------------
    ez_ilda_frame_set(const ez_ilda_frame_set& f)
                     : file_name          (f.file_name         ),
                       num_2D_frames      (f.num_2D_frames     ),
                       num_3D_frames      (f.num_3D_frames     ),
                       frame_index        (f.frame_index       ),
                       frame_select_start (f.frame_select_start),
                       frame_select_end   (f.frame_select_end  ),
                       error              (f.error             ),
                       null_frame         (NULL_frame()        )
                        {
                            insert(begin(), f.begin(), f.end());
                        }
    //-------------------------------------------------------------------------------
    ez_ilda_frame_set(string file)
                     : file_name          (file         ),
                       num_2D_frames      ( 0           ),
                       num_3D_frames      ( 0           ),
                       frame_index        ( 0           ),
                       frame_select_start (-1           ),
                       frame_select_end   (-1           ),
                       error              ( EZ_ILDA_OK  )
                        {
                            from_file(file);
                        }
    //-------------------------------------------------------------------------------
    ez_ilda_frame_set(ez_ilda_point(*F)(int, int), int points_per_frame, int num_frames);
    //-------------------------------------------------------------------------------
   ~ez_ilda_frame_set()    {    clear();    }
    //-------------------------------------------------------------------------------
    inline void set_frame_index(int index) //sets frame_index
                            {
                                if(number_of_frames())
                                {
                                    if(index >= number_of_frames())
                                        index = 0;
                                    if(index < 0)
                                        index = number_of_frames() - 1;
                                    frame_index = index;
                                }
                                else
                                    frame_index = 0;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& frame(int index) //sets frame_index
                            {
                                if(number_of_frames())
                                {
                                    if(index >= number_of_frames())
                                        index = 0;
                                    if(index < 0)
                                        index = number_of_frames() - 1;
                                    frame_index = index;
                                    return at(frame_index);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& next_frame() //sets frame_index
                            {
                                if(number_of_frames())
                                {
                                    frame_index++;
                                    if(frame_index >= number_of_frames())
                                        frame_index = 0;
                                    return at(frame_index);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& previous_frame() //sets frame_index
                            {
                                if(number_of_frames())
                                {
                                    frame_index--;
                                    if(frame_index < 0)
                                        frame_index = number_of_frames() - 1;
                                    return at(frame_index);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame  frame_picker(int index) const
                            {
                                if(number_of_frames())
                                {
                                    if(index < 0)
                                        index = number_of_frames() - 1;
                                    else if(index >= number_of_frames())
                                        index = 0;
                                    return at(index);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& current_frame()
                            {
                                if(number_of_frames())
                                    return at(frame_index);
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline void  delete_current_frame()
                            {
                                if(number_of_frames() > 1)
                                {
                                    erase(begin() + frame_index);
                                    frame_index--;
                                    if(frame_index < 0)
                                        frame_index = 0;
                                }
                                else
                                {
                                    clear();
                                    push_back(null_frame);
                                }
                                return;
                            }
    //-------------------------------------------------------------------------------
    inline void  delete_frame(int index)
                            {
                                if(number_of_frames() > 1)
                                {
                                    if(index < number_of_frames())
                                    {
                                        erase(begin() + index);
                                        if(frame_index >= index)
                                        {
                                            frame_index--;
                                            if(frame_index < 0)
                                                frame_index = number_of_frames() - 1;
                                        }
                                    }
                                }
                                else
                                {
                                    clear();
                                    push_back(null_frame);
                                }
                                return;
                            }
    //-------------------------------------------------------------------------------
    inline void  push_back_frame(const ez_ilda_frame& frame)
                            {
                                push_back(frame);
                                return;
                            }
    //-------------------------------------------------------------------------------
    inline int   total_points() const
                            {
                                int  i,
                                     points = 0;
                                for(i = 0; i < number_of_frames(); i++)
                                    points += (at(i).number_of_points());
                                return points;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& operator ++ () //pre
                            {
                                if(number_of_frames())
                                {
                                    frame_index++;
                                    if(frame_index >= number_of_frames())
                                        frame_index = 0;
                                    return at(frame_index);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& operator -- () // pre
                            {
                                if(number_of_frames())
                                {
                                    frame_index--;
                                    if(frame_index < 0)
                                        frame_index = number_of_frames() - 1;
                                    return at(frame_index);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& operator ++ (int = 0) //post
                            {
                                if(number_of_frames())
                                {
                                    int frame_return = frame_index;
                                    frame_index++;
                                    if(frame_index >= number_of_frames())
                                        frame_index = 0;
                                    return at(frame_return);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& operator -- (int = 0) // post
                            {
                                if(number_of_frames())
                                {
                                    int frame_return = frame_index;
                                    frame_index--;
                                    if(frame_index < 0)
                                        frame_index = number_of_frames() - 1;
                                    return at(frame_return);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame_set& operator  = (const ez_ilda_frame_set& ild)
                            {
                                clear();
                                insert(begin(), ild.begin(), ild.end());
                                file_name     = ild.file_name     ;
                                num_2D_frames = ild.num_2D_frames ;
                                num_3D_frames = ild.num_3D_frames ;
                                renumber();
                                return *this;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame_set& operator += (const ez_ilda_frame_set& ild)
                            {
                                insert(end(), ild.begin(), ild.end());
                                renumber();
                                return *this;
                            }
    //-------------------------------------------------------------------------------
    ez_ilda_frame_set& operator += (const ez_ilda_frame& frame);
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame_set& operator += (ez_ilda_segment& segment)
                            {
                                ez_ilda_header header;
                                header.format   = segment.is_2D();
                                header.quantity = (u_short)segment.number_of_points();
                                push_back(ez_ilda_frame(segment, header));
                                renumber();
                                return *this;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame_set selected_frames()
                            {
                                int                i;
                                ez_ilda_frame_set  selected;
                                for(i = 0; i < number_of_frames(); i++)
                                    if(at(i).is_selected)
                                        selected += at(i);
                                return selected;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame_set  operator +  (const ez_ilda_frame_set& ild) const
                            {
                                ez_ilda_frame_set sum(*this);
                                sum += ild;
                                return sum;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame& operator [] (int index) // does not set frame_index
                            {
                                if(number_of_frames())
                                {
                                    if(index >= number_of_frames())
                                        index = 0;
                                    if(index < 0)
                                        index = number_of_frames() - 1;
                                    return at(index);
                                }
                                return null_frame;
                            }
    //-------------------------------------------------------------------------------
    inline ez_ilda_frame sum_of_frames()
                            {
                                bool           same_palettes = true;
                                int            i;
                                ez_ilda_frame  frame(at(0).palette_index);
                                for(i = 1; i < number_of_frames(); i++)
                                    if(frame.palette_index != at(i).palette_index)
                                    {
                                        same_palettes = false;
                                        break;
                                    }
                                if(!same_palettes)
                                {
                                    frame.promote_to_true_color();
                                    for(i = 0; i < number_of_frames(); i++)
                                        at(i).promote_to_true_color();
                                }
                                for(i = 0; i < number_of_frames(); i++)
                                    frame += at(i);
                                return frame;
                            }
    //-------------------------------------------------------------------------------
    int                 number_of_frames    () const {   return (int)size()   ;   }
    int                 number_of_2D_frames () const {   return num_2D_frames ;   }
    int                 number_of_3D_frames () const {   return num_3D_frames ;   }
    //-------------------------------------------------------------------------------
    ez_ilda_frame_set&  reverse                   (                        )       ;
    ez_ilda_frame_set&  frame_reverse             (                        )       ;
    ez_ilda_frame_set&  superimpose_from_file     (string    file          )       ;
    int                 from_file                 (string    file,
                                                   bool      append = false)       ;
    int                 from_ifstream             (ifstream& in,
                                                   bool      append = false)       ;
    int                 from_wave_file            (string    file,
                                                   bool      append = false)       ;
    int                 from_ifstream_wave        (ifstream& in,
                                                   bool      append = false)       ;
    int                 from_dxf_file             (string    file,
                                                   bool      append = false)       ;
    int                 from_dxf_directory        (string    dir,
                                                   bool      append = false)       ;
    int                 from_ifstream_dxf         (ifstream& in,
                                                   bool      append = false)       ;
    int                 from_shp_file             (string    file          )       ;
    int                 from_ifstream_shp         (ifstream& in            )       ;
    void                flatten_z                 (                        )       ;
    void                z_order_points            (unsigned short span     )       ;
    void                unblank_all_points        (                        )       ;
    void                minimize_blank_points     (                        )       ;
    void                remove_dwell_points       (                        )       ;
    void                minimize_vectors          (                        )       ;
    void                minimize_points           (                        )       ;
    void                explode_current_frame     (                        )       ;
    void                collapse_selected_frames  (                        )       ;
    void                fracture_segments         (                        )       ;
    void                bond_segments             (                        )       ;
    void                reorder_segments          (                        )       ;
    void                add_dwell_points          (                        )       ;
    void                add_span_points           (                        )       ;
    void                delete_redundant_frames   (                        )       ;
    void                optimize_points           (                        )       ;
    void                rainbow_recolor           (int effect, float factor)       ;
    void                quarter_turn              (int plane, int turns    )       ;
    int                 scale_around_origin       (ez_3D_float f           )       ;
    int                 scale_around_origin       (                        )       ;
    int                 rotate_around_origin      (ez_3D_float f           )       ;
    int                 rotate_around_origin      (                        )       ;
    int                 move                      (ez_3D_float f           )       ;
    int                 move                      (                        )       ;
    void                toggle_frame_select_start (                        )       ;
    void                toggle_frame_select_end   (                        )       ;
    void                delete_selected_frames    (                        )       ;
    void                trim_to_selected_frames   (                        )       ;
    void                bit_reduce_to_palette     (                        )       ;
    void                promote_to_true_color     (                        )       ;
    void                best_match_palette        (int index               )       ;
    void                best_match_target_palette (                        )       ;
    void                accept_palette            (int index               )       ;
    void                accept_target_palette     (                        )       ;
    void                color_over_ezfb_bitmap    (struct ezfb_bitmap* bmp )       ;
    void                color_over_ezfb_bitmap    (string file             )       ;
    void                flip                      (int plane               )       ;
    void                renumber                  (                        )       ;
    void                save_as_ilda              (string file             )       ;
    void                to_ofstream               (ofstream& out           )       ;
    void                print_ez_ilda_error       (int error               ) const ;
    void                display_stats             (                        ) const ;
    int                 total_wave_samples        (                        ) const ;
    void                save_as_wave              (string file             ) const ;
    void                to_ofstream_wave          (ofstream& out           ) const ;
    void                save_as_bmp_directory     (string dir              ) const ;
    void                save_as_dxf_directory     (string dir              ) const ;
    void                save_as_code              (string object_name      )       ;
    //-------------------------------------------------------------------------------
    string              file_name          ;
    int                 num_2D_frames      ,
                        num_3D_frames      ,
                        frame_index        ,
                        frame_select_start ,
                        frame_select_end   ,
                        error              ;
    ez_ilda_frame       null_frame         ;
    ez_ilda_segment     intro              ;
};

//###################################################################################
#endif

//###################################################################################
/////////////////////////////////////////////////////////////////////////////////////
//###################################################################################
